@@ -36,10 +36,12 @@ struct _GtkHTMLPrivate {
 	GtkHTMLFontStyle insertion_font_style;
 
 	gboolean update_styles;
+	gboolean skip_update_cursor;
+	gboolean cursor_moved;
 
 	gint selection_type;
 
-	char  *base_url;
+	gchar  *base_url;
 
 	GtkWidget *search_input_line;
 
@@ -76,7 +78,7 @@ struct _GtkHTMLPrivate {
 
 	gboolean in_key_binding;
 
-	char *caret_first_focus_anchor;
+	gchar *caret_first_focus_anchor;
 };
 
 void  gtk_html_private_calc_scrollbars  (GtkHTML                *html,
@@ -98,14 +100,13 @@ gchar *gtk_html_filename_from_uri	(const gchar		*uri);
 
 #ifdef G_OS_WIN32
 
-const char *_get_icondir (void) G_GNUC_CONST;
-const char *_get_gtkhtml_datadir (void) G_GNUC_CONST;
-const char *_get_localedir (void) G_GNUC_CONST;
-const char *_get_glade_datadir (void) G_GNUC_CONST;
-const char *_get_prefix (void) G_GNUC_CONST;
-const char *_get_sysconfdir (void) G_GNUC_CONST;
-const char *_get_datadir (void) G_GNUC_CONST;
-const char *_get_libdir (void) G_GNUC_CONST;
+const gchar *_get_icondir (void) G_GNUC_CONST;
+const gchar *_get_gtkhtml_datadir (void) G_GNUC_CONST;
+const gchar *_get_localedir (void) G_GNUC_CONST;
+const gchar *_get_prefix (void) G_GNUC_CONST;
+const gchar *_get_sysconfdir (void) G_GNUC_CONST;
+const gchar *_get_datadir (void) G_GNUC_CONST;
+const gchar *_get_libdir (void) G_GNUC_CONST;
 
 #undef ICONDIR
 #define ICONDIR _get_icondir ()
@@ -116,9 +117,6 @@ const char *_get_libdir (void) G_GNUC_CONST;
 #undef GNOMELOCALEDIR
 #define GNOMELOCALEDIR _get_localedir ()
 
-#undef GLADE_DATADIR
-#define GLADE_DATADIR _get_glade_datadir ()
-
 #undef PREFIX
 #define PREFIX _get_prefix ()
 
